@@ -7,10 +7,12 @@ function TLD() {
 
 TLD.prototype = {
   get: function get(key1, key2) {
-    if (this.storage[key1] == undefined)
+    if (this.storage[key1] == undefined) {
       return undefined;
-    if (key2 == undefined)
+    }
+    if (key2 == undefined) {
       return this.storage[key1];
+    }
     return this.storage[key1][key2];
   },
 
@@ -37,7 +39,7 @@ TLD.prototype = {
       this.set(key1, key2, res);
     }
     return res;
-  },
+  }
 };
 
 dynamicforms = {
@@ -46,7 +48,7 @@ dynamicforms = {
     'template':          'dynamicforms/bootstrap/',
     'jquery_ui':         false,
     'edit_in_dialog':    true,
-    'bootstrap_version': 'v4',
+    'bootstrap_version': 'v4'
   },
 
   /**
@@ -65,7 +67,7 @@ dynamicforms = {
    * @param progressSettings: Progress dialog settings - for custom progress dialogs.
    */
   showProgressDlg: function showProgressDialog(progressDlgID, progressSettings) {
-    var zIndexElement = null;
+    var zIndexElement;
     var progressDlg   = $('#' + progressDlgID);
     if (dynamicforms.DYNAMICFORMS.jquery_ui) {
       if (progressSettings === undefined)
@@ -108,7 +110,7 @@ dynamicforms = {
     }
     var zIndexOrig = zIndexElement.css('z-index');
     var zIndex     = parseInt(zIndexOrig);
-    if (zIndex == NaN)
+    if (isNaN(zIndex))
       zIndex = zIndexOrig;
     else {
       // Because all bootstrap modules have same z-index. If we want that clickable progress bar will be on top of all
@@ -133,7 +135,7 @@ dynamicforms = {
       dynamicforms.progressCheckInterval = null;
     } else {
       $.ajax({url: '/dynamicforms/progress/', headers: {'X-DF-TIMESTAMP': timestamp}})
-        .done(function (data, textStatus, jqXHR) {
+        .done(function (data) {
           var pb_indet   = $('#df-progress-bar-indeterminate');
           var pb_det     = $('#df-progress-bar-determinate');
           var percent    = data.value;
@@ -155,7 +157,7 @@ dynamicforms = {
             }
           }
         })
-        .fail(function (jqXHR, textStatus, errorThrown) {
+        .fail(function () {
           if (dynamicforms.shouldShowProgressDlg && !dynamicforms.progressDlgStartShowing) {
             dynamicforms.showProgressDlg(progressDlgID, progressSettings);
           }
@@ -311,11 +313,11 @@ dynamicforms = {
                                       data:        data,
                                       dataType:    dataType,
                                       headers:     headers,
-                                      traditional: true,
+                                      traditional: true
                                     }
                                   })
       .done(doneFuncExec)
-      .fail(function (xhr, status, error) {
+      .fail(function (xhr) {
         // TODO: this doesn't handle errors correctly: if return status is 400 something, it *might* be OK
         //  but if it's 500 something, dialog will be replaced by non-dialog code and displaying it will fail
         //  also for any authorization errors, CSRF, etc, it will again fail
@@ -580,7 +582,7 @@ dynamicforms = {
     if (dynamicforms.DYNAMICFORMS.edit_in_dialog) {
       var ajaxSetts = {
         url:     recordURL,
-        headers: {'X-DF-RENDER-TYPE': 'dialog'},
+        headers: {'X-DF-RENDER-TYPE': 'dialog'}
       }
       if (params != undefined && params.data != undefined)
         ajaxSetts['data'] = params.data;
@@ -627,7 +629,7 @@ dynamicforms = {
       // Count how many lines should "No data" element span
       var colCount      = $("th").length;
       var noDataElement = "<tr data-title='NoData'><td colspan=" + colCount + " style='text-align: center'>No data</td></tr>";
-      var $tblBody      = $("tbody").append(noDataElement);
+      $("tbody").append(noDataElement);
     }
   },
 
@@ -644,7 +646,7 @@ dynamicforms = {
                                     ajax_setts: {
                                       url:     recordURL,
                                       method:  'DELETE',
-                                      headers: {'X-CSRFToken': dynamicforms.csrf_token},
+                                      headers: {'X-CSRFToken': dynamicforms.csrf_token}
                                     }
                                   })
       .done(function (dialogHTML) {
@@ -662,8 +664,7 @@ dynamicforms = {
         } else if (typeof (refreshType) == 'function') {
           refreshType();
         } else if (refreshType.indexOf('redirect') !== -1) {
-          var redirectUrl      = refreshType.split(':').pop();
-          window.location.href = redirectUrl;
+          window.location.href = refreshType.split(':').pop();
         } else if (refreshType == 'no refresh') {
           // pass
         } else if (dynamicforms.isFunction(refreshType)) {
@@ -1130,10 +1131,18 @@ dynamicforms = {
 
       if (link_params != undefined) {
         link_params = link_params.split('&');
-        var addfilter = '';
+        var addfilter = '',
+            filterFields = dynamicforms.filterFields(formID);
+        filterFields.push('cursor');
+        filterFields.push('ordering');
         for (var i = 0; i < link_params.length; i++) {
-          if (link_params[i].substr(0, 6) == 'cursor')
-            continue;
+          var skip_link = false;
+          for (var j = 0; j < filterFields.length; j++) {
+            var cmpVal = filterFields[j] + '='
+            if (link_params[i].substr(0, cmpVal.length) == cmpVal)
+              skip_link = true;
+          }
+          if (skip_link) continue;
           addfilter += '&' + link_params[i];
         }
         if (filter != 'nofilter') {
@@ -1165,7 +1174,7 @@ dynamicforms = {
       $.ajax({
                type:    'GET',
                headers: {'X-CSRFToken': dynamicforms.csrf_token, 'X-DF-RENDER-TYPE': 'table rows'},
-               url:     link_next,
+               url:     link_next
              }).done(function (data) {
 
         data                     = $(data).filter("tr");
@@ -1223,6 +1232,14 @@ dynamicforms = {
     })
   },
 
+  filterFields: function filterFields(formID) {
+    var res = [];
+    $("#list-" + formID).find(".dynamicforms-filterrow th").each(function () {
+      res.push($(this).attr("data-name"));
+    });
+    return res;
+  },
+
   /**
    * Prepares filter string and calls server to get filtered data
    *
@@ -1232,7 +1249,9 @@ dynamicforms = {
   filterData: function filterData(formID, returnDict) {
     if (returnDict == undefined)
       returnDict = false;
-    var filter = {};
+    var filter = {},
+        order = dynamicforms.df_tbl_pagination.get(formID, 'ordering');
+
     $("#list-" + formID).find(".dynamicforms-filterrow th").each(function (index) {
 
       var element = $(this).find("[name='" + $(this).attr("data-name") + "']");
@@ -1255,8 +1274,12 @@ dynamicforms = {
     if (returnDict)
       return filter;
     filter = jQuery.param(filter);
-    if (!filter.length)
-      filter = 'nofilter';
+    if (!filter.length) {
+      if (order != null && order != '') filter = 'ordering=' + order;
+      else filter = 'nofilter';
+    }
+    else if (order != null && order != '') filter += '&ordering=' + order;
+
     dynamicforms.paginatorGetNextPage(formID, filter);
   },
 
@@ -1303,8 +1326,156 @@ dynamicforms = {
     });
     $select2.select2("destroy").select2(sel2Conf);
     $select2.data('sel2Conf', sel2Conf);
-  }
+  },
 
+  orderingColumnClicked: function orderingColumnClicked(event, $col, formID) {
+    var ordering_index = dynamicforms.getOrderingAndIndex($col),
+        index = ordering_index.index - 1,
+        ordering = dynamicforms.getCurrentOrder(formID).segments;
+
+    if (!$col.hasClass('ordering')) return;  // Only process columns that are sortable
+
+    event.stopPropagation();
+
+    function renumber() {
+      for (var i = 0; i < ordering.length; i++)
+        ordering[i].index = i + 1;
+    }
+
+    if (event.altKey) {
+      // Show dialog with sort order options
+    }
+    else if (event.ctrlKey && event.shiftKey) {
+      // remove segment from ordering
+      if (index >= 0) {
+        ordering.splice(index, 1);
+        renumber();
+        dynamicforms.setCurrentOrder(formID, ordering);
+      }
+    }
+    else if (event.ctrlKey) {
+      // Move the segment to top
+      if (index >= 0) ordering.splice(index, 1);
+      if (ordering_index.direction === null) ordering_index.direction = true;
+      ordering.splice(0, 0, ordering_index);
+      renumber();
+      dynamicforms.setCurrentOrder(formID, ordering);
+    }
+    else if (event.shiftKey) {
+      // Change segment sort direction (and add it to sort segments list if not already there)
+      // if shift is pressed add segment to existing ones. if not, set this column as the only segment of sort
+      if (ordering_index.index < 1) {
+        ordering_index.direction = ordering_index.direction !== false;
+        ordering_index.index = ordering.length + 1;
+        ordering.push(ordering_index);
+      }
+      else {
+        ordering[index].direction = ordering[index].direction === false;
+      }
+      dynamicforms.setCurrentOrder(formID, ordering);
+    }
+    else {
+      ordering_index.index = 1;
+      if (ordering_index.direction === null) ordering_index.direction = true;
+      else if (ordering_index.direction === true) ordering_index.direction = false;
+      else {
+        ordering_index.direction = null;
+        ordering_index.index = 0;
+      }
+      dynamicforms.setCurrentOrder(formID, [ordering_index]);
+    }
+  },
+
+  getOrderingAndIndex: function getOrderingAndIndex($col) {
+    var dir = null,
+        index = 0,
+        fieldName = null;
+    if ($col.hasClass('ordering')) {
+      fieldName = $col.attr('data-field-name');
+      if ($col.hasClass('asc')) dir = true;  // ascending
+      else if ($col.hasClass('desc')) dir = false;  // ascending
+      for (var i = 1; i <= 20; i++) {
+        if ($col.hasClass('seg-' + i)) {
+          index = i;
+          break;
+        }
+      }
+    }
+    return {direction: dir, index: index, fieldName: fieldName};
+  },
+
+  renderOrderingIndicator: function renderOrderingIndicator($col, ordering_index) {
+    var dir = '\u2195',
+        index = '';
+
+    if ($col.hasClass('ordering')) {
+      if (ordering_index.direction === true) dir = '\u2191';  // ascending
+      else if (ordering_index.direction === false) dir = '\u2193';  // ascending
+      if (ordering_index.index) index = String.fromCharCode(0x2460 + ordering_index.index - 1);
+      $col.find('span.ordering').text(dir + index);
+    }
+  },
+
+  setCurrentOrder: function setCurrentOrder(formID, ordering) {
+    var order = {};
+    for (var i = 0; i < ordering.length; i++)
+      order[ordering[i].fieldName] = ordering[i];
+
+    $('#list-' + formID + '>thead>tr>th').each(function() {
+      var $col = $(this),
+          fieldName = $col.attr('data-field-name');
+      if ($col.hasClass('ordering')) {
+        var ordr = order[fieldName];
+        if (ordr == null || ordr.index < 1) $col.attr('class', 'ordering unsorted');
+        else $col.attr('class',
+          ('ordering ' + (ordr.direction === true ? 'asc' : 'desc') + (' seg-' + ordr.index))
+        );
+      }
+    });
+    dynamicforms.getInitialOrdering(formID, false);
+    dynamicforms.filterData(formID);
+  },
+
+  getCurrentOrder: function getCurrentOrder(formID) {
+    var ordering = [],
+        order = [];
+    $('#list-' + formID + '>thead>tr>th').each(function() {
+      var $col = $(this);
+      if ($col.hasClass('ordering')) {
+        var ordering_index = dynamicforms.getOrderingAndIndex($col);
+        if (ordering_index.index > 0) ordering.push(ordering_index);
+      }
+    });
+    function cmp(a, b) {
+      if (a.index > b.index) return 1;
+      else if (a.index < b.index) return -1;
+      return 0;
+    }
+    ordering.sort(cmp);
+    for (var i = 0; i < ordering.length; i++) {
+      if (ordering[i].direction === true)
+        order.push(ordering[i].fieldName);
+      else
+        order.push('-' + ordering[i].fieldName);
+    }
+    return {order: order.join(','), segments: ordering};
+  },
+
+  getInitialOrdering: function getInitialOrdering(formID, assignOnClickListener) {
+    $('#list-' + formID + '>thead>tr>th').each(function() {
+      var $col = $(this);
+      if ($col.hasClass('ordering')) {
+        var ordering_index = dynamicforms.getOrderingAndIndex($col);
+        dynamicforms.renderOrderingIndicator($col, ordering_index);
+        if (assignOnClickListener !== false) {
+          $col.on('click', function colclick(evt) {
+            dynamicforms.orderingColumnClicked(evt, $col, formID);
+          });
+        }
+      }
+    });
+    dynamicforms.df_tbl_pagination.set(formID, 'ordering', dynamicforms.getCurrentOrder(formID).order);
+  }
 };
 
 $(document).ready(function () {
